"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslint_1 = require("tslint");
const TscFormatter_1 = require("./TscFormatter");
const debugTools_1 = require("../helpers/debugTools");
class TslintRunner {
    constructor(tslintArgs, doneCb) {
        this.running = false;
        this.terminated = false;
        this.result = "";
        this.autofix = false;
        this.startLintingWhenNotRunning = () => {
            // Make sure to wait until tslint is really done before firing a new tslint task
            if (this.running)
                setImmediate(this.startLintingWhenNotRunning);
            else {
                this.terminated = false;
                this.lintFilesAsync();
            }
        };
        this.tslintCfg = tslintArgs.rulesFile;
        this.tsconfig = tslintArgs.tsconfigPath;
        this.autofix = tslintArgs.autoFix;
        this.doneCb = doneCb;
    }
    startLinting() {
        debugTools_1.debugLog("Tslint: Starting  with options", {
            tslintCfg: this.tslintCfg,
            tsconfig: this.tsconfig,
            autofix: this.autofix
        });
        this.startLintingWhenNotRunning();
    }
    terminate() {
        if (this.running) {
            debugTools_1.debugLog("Tslint: Aborting", this.tsconfig);
        }
        this.terminated = true;
    }
    getLastResult() {
        return this.result;
    }
    isRunning() {
        return this.running;
    }
    lintFilesAsync() {
        const program = tslint_1.Linter.createProgram(this.tsconfig);
        const linter = new tslint_1.Linter({ fix: this.autofix, formatter: TscFormatter_1.Formatter }, program);
        const files = tslint_1.Linter.getFileNames(program);
        const configuration = tslint_1.Configuration.findConfiguration(this.tslintCfg).results;
        const lintInEventLoop = (remainingFiles) => {
            const file = remainingFiles.shift();
            const fileContents = program.getSourceFile(file).getFullText();
            linter.lint(file, fileContents, configuration);
            if (remainingFiles.length && !this.terminated)
                setImmediate(lintInEventLoop, remainingFiles);
            else {
                this.running = false;
                if (this.terminated)
                    debugTools_1.debugLog("Tslint: Aborted", this.tsconfig);
                else {
                    this.result = linter.getResult().output;
                    debugTools_1.debugLog("Tslint: done and printing for", this.tsconfig);
                    this.doneCb();
                }
            }
        };
        this.running = true;
        lintInEventLoop(files);
    }
}
exports.TslintRunner = TslintRunner;
